@@ -3,10 +3,8 @@ package main
 import (
 	"bytes"
 	"fmt"
-	"io/ioutil"
 	"os"
 	"reflect"
-	"sort"
 	"strings"
 	"testing"
 
@@ -14,6 +12,10 @@ import (
 	"zgo.at/zstd/ztest"
 )
 
+func init() {
+	isTerm = false
+}
+
 func TestCLI(t *testing.T) {
 	tests := []struct {
 		in   []string
@@ -123,10 +125,13 @@ func TestSearch(t *testing.T) {
 		{[]string{"-q", "s", "floral", "bullet"}, "HEART", 2, -1},
 		{[]string{"-q", "s", "rightwards arrow", "heavy"}, "HEAVY", 16, -1},
 
-		{[]string{"-qo", "s", "floral", "bullet"}, "WHITE BULLET", 15, -1},
+		{[]string{"-qo", "s", "floral", "bullet"}, "WHITE BULLET", 16, -1},
 
 		{[]string{"s", "nomatch_nomatch"}, "no matches", 1, 1},
 		{[]string{"-q", "s", "nomatch_nomatch"}, "", 0, 1},
+
+		// factorial from aliases
+		{[]string{"-q", "s", "factorial"}, "EXCLAMATION MARK", 1, -1},
 	}
 
 	for _, tt := range tests {
@@ -166,6 +171,7 @@ func TestPrint(t *testing.T) {
 		{[]string{"-q", "p", "U+2042..2044"}, "ASTERISM", 3, -1},
 		{[]string{"-q", "p", "2042..U+2044"}, "ASTERISM", 3, -1},
 		{[]string{"-q", "p", "0x2042..0o20104"}, "ASTERISM", 3, -1},
+		{[]string{"-q", "p", "%x2042..%x2044"}, "ASTERISM", 3, -1},
 		{[]string{"-q", "p", "0b10000001000010..0o20104"}, "ASTERISM", 3, -1},
 		{[]string{"-q", "p", "0b10000001000010-0o20104"}, "ASTERISM", 3, -1},
 		{[]string{"p", "9999999999"}, `out of range: "9999999999"`, 1, 1},
@@ -174,7 +180,7 @@ func TestPrint(t *testing.T) {
 		{[]string{"-q", "p", "0d90"}, "CAPITAL LETTER Z", 1, -1},
 		{[]string{"-q", "p", "0D90"}, "CAPITAL LETTER Z", 1, -1},
 
-		{[]string{"p", ""}, `invalid codepoint: not a number or codepoint: ""`, 1, 1},
+		{[]string{"p", "lol"}, `invalid codepoint: not a number or codepoint: "lol"`, 1, 1},
 		{[]string{"p", "nonsense"}, `invalid codepoint: not a number or codepoint: "nonsense"`, 1, 1},
 		{[]string{"p", "2042..xxx"}, `invalid codepoint: not a number or codepoint: "xxx"`, 1, 1},
 		{[]string{"p", "xxx..xxx"}, `invalid codepoint: not a number or codepoint: "xxx"`, 1, 1},
@@ -185,7 +191,7 @@ func TestPrint(t *testing.T) {
 		{[]string{"-q", "p", "OtherPunctuation"}, "ASTERISM", 628, -1},
 		{[]string{"-q", "p", "Po"}, "ASTERISM", 628, -1},
 		{[]string{"-q", "p", "GeneralPunctuation"}, "ASTERISM", 111, -1},
-		{[]string{"-q", "p", "all"}, "ASTERISM", 34924, -1},
+		{[]string{"-q", "p", "all"}, "ASTERISM", 34931, -1},
 
 		{[]string{"-q", "-r", "p", "U9"}, "'\t'", 1, -1},
 
@@ -218,6 +224,9 @@ func TestPrint(t *testing.T) {
 		{[]string{"-q", "p", "U+DB80"}, "<Private Use High Surrogate, First>", 1, -1},     // <Private Use High Surrogate> (First)
 		{[]string{"-q", "p", "U+DB81"}, "<Private Use High Surrogate>", 1, -1},            // <Private Use High Surrogate>
 		{[]string{"-q", "p", "U+DBFF"}, "<Private Use High Surrogate, Last>", 1, -1},      // <Private Use High Surrogate> (Last)
+
+		// Allow trailing commas, slashes.
+		{[]string{"-q", "p", "/U+2109,", ",", "/", ",U+2131/"}, "SCRIPT CAPITAL F", 2, -1},
 	}
 
 	for _, tt := range tests {
@@ -259,7 +268,7 @@ func TestEmoji(t *testing.T) {
 		{[]string{"e", "-q", "group:hands"},
 			[]string{"👏", "🙌", "🫶", "👐", "🤲", "🤝", "🙏"}},
 		{[]string{"e", "-q", "-tone", "dark", "g:hands"},
-			[]string{"👏🏿", "🙌🏿", "🫶🏿", "👐🏿", "🤲🏿", "🤝", "🙏🏿"}},
+			[]string{"👏🏿", "🙌🏿", "🫶🏿", "👐🏿", "🤲🏿", "🤝🏿", "🙏🏿"}},
 
 		{[]string{"e", "-q", "shrug"},
 			[]string{"🤷"}},
@@ -319,61 +328,48 @@ func TestEmoji(t *testing.T) {
 	}
 }
 
-func TestAllEmoji(t *testing.T) {
-	exit, _, outbuf := zli.Test(t)
-	os.Args = append([]string{"testuni"}, []string{"e", "-q", "-gender", "all", "-tone", "all", "all"}...)
-
-	func() {
-		defer exit.Recover()
-		main()
-	}()
-
-	/*
-		grep -v '^#' unidata/.cache/emoji-test.txt |
-		    grep fully-qualified |
-		    grep -Ev '(holding hands|handshake|kiss:|couple with heart).*tone' |
-		    grep -Eo '# .+? E[0-9]' |
-		    cut -d ' ' -f2 >| testdata/emojis
-
-		double tones: 70
-		family: 145
-	*/
-	w, err := ioutil.ReadFile("./testdata/emojis")
-	if err != nil {
-		t.Fatal(err)
+func TestFormat(t *testing.T) {
+	tests := []struct {
+		in   []string
+		want []string
+	}{
+		// -f q:
+		// TODO: using %(col q:()) doesn't work, as it parses that ) as the
+		// closing of the column.
+		{[]string{"e", "-q", "-f", "%(cldr q:[:])", "orange heart"},
+			[]string{"[]"}},
+		{[]string{"e", "-q", "-f", "%(cldr Q:[:])", "orange heart"},
+			[]string{""}},
+		{[]string{"e", "-q", "-f", "%(cldr q:[:])", "red heart"},
+			[]string{"[emotion, love]"}},
+		{[]string{"e", "-q", "-f", "%(cldr Q:[:])", "red heart"},
+			[]string{"[emotion, love]"}},
 	}
-	wantEmojis := strings.Split(strings.TrimSpace(string(w)), "\n")
+	for _, tt := range tests {
+		t.Run(strings.Join(tt.in, "_"), func(t *testing.T) {
+			_, _, outbuf := zli.Test(t)
+			os.Args = append([]string{"testuni"}, tt.in...)
 
-	out := strings.Split(strings.TrimRight(outbuf.String(), "\n"), "\n")
-	outEmojis := make([]string, len(out))
-	for i := range out {
-		outEmojis[i] = out[i][:strings.Index(out[i], " ")]
-	}
+			main()
 
-	if len(outEmojis) != len(wantEmojis) {
-		t.Errorf("all: wrong length: want %d, got %d\n", len(wantEmojis), len(outEmojis))
-	}
+			var out []string
+			for _, line := range strings.Split(strings.TrimSpace(outbuf.String()), "\n") {
+				line = strings.ReplaceAll(line, "\t", " ")
+				out = append(out, line)
+			}
 
-	// Still some \ufe0f issues
-	return
-	t.Skip()
+			for i := range tt.want {
+				tt.want[i] = strings.Replace(tt.want[i], "Z", "\u200d", -1)
+				tt.want[i] = strings.Replace(tt.want[i], "S", "\ufe0f", -1)
+			}
 
-	// TODO: this shouldnt; be needed
-	sort.Strings(wantEmojis)
-	sort.Strings(outEmojis)
-
-	for i := range wantEmojis {
-		wantEmojis[i] = strings.ReplaceAll(wantEmojis[i], "\ufe0f", `\ufe0f`)
+			if !reflect.DeepEqual(out, tt.want) {
+				a := strings.ReplaceAll(fmt.Sprintf("%#v", out), "\ufe0f", `\ufe0f`)
+				b := strings.ReplaceAll(fmt.Sprintf("%#v", tt.want), "\ufe0f", `\ufe0f`)
+				t.Errorf("wrong output\nout:  %s\nwant: %s", a, b)
+			}
+		})
 	}
-	for i := range outEmojis {
-		outEmojis[i] = strings.ReplaceAll(outEmojis[i], "\ufe0f", `\ufe0f`)
-	}
-	if !reflect.DeepEqual(outEmojis, wantEmojis) {
-		t.Errorf("emoji lists not equal\nout:  %v\nwant: %v", outEmojis, wantEmojis)
-	}
-	//if d := ztest.Diff(outEmojis, wantEmojis); d != "" {
-	//	t.Error(d)
-	//}
 }
 
 func TestJSON(t *testing.T) {
@@ -383,28 +379,31 @@ func TestJSON(t *testing.T) {
 	main()
 
 	want := ` [{
-	"bin": "10000010101100",
-	"block": "Currency Symbols",
-	"cat": "Currency_Symbol",
-	"char": "€",
-	"cpoint": "U+20AC",
-	"dec": "8364",
+	"aliases": "",
+	"bin":     "10000010101100",
+	"block":   "Currency Symbols",
+	"cat":     "Currency_Symbol",
+	"cells":   "1",
+	"char":    "€",
+	"cpoint":  "U+20AC",
+	"dec":     "8364",
 	"digraph": "=e",
-	"hex": "20ac",
-	"html": "&euro;",
-	"json": "\\u20ac",
-	"keysym": "EuroSign",
-	"name": "EURO SIGN",
-	"oct": "20254",
-	"plane": "Basic Multilingual Plane",
-	"props": "",
-	"script": "Common",
+	"hex":     "20ac",
+	"html":    "&euro;",
+	"json":    "\\u20ac",
+	"keysym":  "EuroSign",
+	"name":    "EURO SIGN",
+	"oct":     "20254",
+	"plane":   "Basic Multilingual Plane",
+	"props":   "",
+	"refs":    "U+20A0",
+	"script":  "Common",
 	"unicode": "2.1",
 	"utf16be": "20 ac",
 	"utf16le": "ac 20",
-	"utf8": "e2 82 ac",
-	"width": "ambiguous",
-	"xml": "&#x20ac;"
+	"utf8":    "e2 82 ac",
+	"width":   "ambiguous",
+	"xml":     "&#x20ac;"
 }]
 `
 	got := outbuf.String()
