@@ -7,6 +7,8 @@ import (
 	"strings"
 	"unicode/utf16"
 	"unicode/utf8"
+
+	"zgo.at/runewidth"
 )
 
 type (
@@ -26,6 +28,12 @@ type (
 		name     string
 	}
 
+	name struct {
+		aliases   []string
+		refs      []rune
+		decompose []rune
+	}
+
 	Width        uint8      // Unicode width
 	Plane        uint8      // Unicode plane
 	Category     uint8      // Unicode category
@@ -204,7 +212,7 @@ func FromString(s string) (Codepoint, error) {
 	s = strings.ToUpper(s)
 	var base = 16
 	switch {
-	case strings.HasPrefix(s, "0X") || strings.HasPrefix(s, "U+"):
+	case strings.HasPrefix(s, "0X") || strings.HasPrefix(s, "U+") || strings.HasPrefix(s, "%X"):
 		s = s[2:]
 	case strings.HasPrefix(s, "0D"):
 		s = s[2:]
@@ -277,6 +285,9 @@ func (c Codepoint) Name() string { return c.name }
 // Width gets this codepoint's width.
 func (c Codepoint) Width() Width { return c.width }
 
+// Cells gets the number of cells this codepoint will display as; 0, 1, or 2.
+func (c Codepoint) Cells() uint8 { return uint8(runewidth.RuneWidth(c.Codepoint)) }
+
 // Category gets this codepoint's category.
 func (c Codepoint) Category() Category { return c.category }
 
@@ -333,6 +344,19 @@ func (c Codepoint) Unicode() Unicode {
 	return c.unicode
 }
 
+func (c Codepoint) Aliases() []string {
+	return names[c.Codepoint].aliases
+}
+
+func (c Codepoint) Refs() []string {
+	r := names[c.Codepoint].refs
+	s := make([]string, 0, len(r))
+	for _, rr := range r {
+		s = append(s, fmt.Sprintf("U+%04X", rr))
+	}
+	return s
+}
+
 // FormatCodepoint formats the codepoint in Unicode notation.
 func (c Codepoint) FormatCodepoint() string {
 	return fmt.Sprintf("U+%04X", c.Codepoint)
